@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -14,7 +14,7 @@
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
@@ -22,18 +22,17 @@
  * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
  *
- *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * either of the GNU General Public License Version 2 or later (the "GPL"),
+ * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -116,6 +115,7 @@ public:
   NS_IMETHOD SetNodeFixup(nsIDocumentEncoderNodeFixup *aFixup);
                                        
 protected:
+  void Initialize();
   nsresult SerializeNodeStart(nsIDOMNode* aNode, PRInt32 aStartOffset,
                               PRInt32 aEndOffset, nsAString& aStr);
   nsresult SerializeToStringRecursive(nsIDOMNode* aNode,
@@ -181,13 +181,19 @@ NS_INTERFACE_MAP_END
 
 nsDocumentEncoder::nsDocumentEncoder()
 {
+  Initialize();
+  mMimeType.AssignLiteral("text/plain");
 
-  mMimeType.Assign(NS_LITERAL_STRING("text/plain"));
+}
 
+void nsDocumentEncoder::Initialize()
+{
   mFlags = 0;
   mWrapColumn = 72;
   mStartDepth = 0;
   mEndDepth = 0;
+  mStartRootIndex = 0;
+  mEndRootIndex = 0;
   mHaltRangeHint = PR_FALSE;
 }
 
@@ -203,6 +209,8 @@ nsDocumentEncoder::Init(nsIDocument* aDocument,
   if (!aDocument)
     return NS_ERROR_INVALID_ARG;
 
+  Initialize();
+
   mDocument = aDocument;
 
   mMimeType = aMimeType;
@@ -880,7 +888,7 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
   aOutputString.Truncate();
 
   nsCAutoString progId(NS_CONTENTSERIALIZER_CONTRACTID_PREFIX);
-  progId.AppendWithConversion(mMimeType);
+  AppendUTF16toUTF8(mMimeType, progId);
 
   mSerializer = do_CreateInstance(progId.get());
   NS_ENSURE_TRUE(mSerializer, NS_ERROR_NOT_IMPLEMENTED);
@@ -952,7 +960,7 @@ nsDocumentEncoder::EncodeToStream(nsIOutputStream* aStream)
                                                    getter_AddRefs(mUnicodeEncoder));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (mMimeType.EqualsIgnoreCase("text/plain")) {
+  if (mMimeType.LowerCaseEqualsLiteral("text/plain")) {
     rv = mUnicodeEncoder->SetOutputErrorBehavior(nsIUnicodeEncoder::kOnError_Replace, nsnull, '?');
     NS_ENSURE_SUCCESS(rv, rv);
   }
@@ -1070,10 +1078,14 @@ nsHTMLCopyEncoder::Init(nsIDocument* aDocument,
   if (!aDocument)
     return NS_ERROR_INVALID_ARG;
 
+  mIsTextWidget = PR_FALSE;
+  Initialize();
+
   mIsCopying = PR_TRUE;
   mDocument = aDocument;
 
-  mMimeType = NS_LITERAL_STRING("text/html");
+
+  mMimeType.AssignLiteral("text/html");
   
   // Make all links absolute when copying
   // (see related bugs #57296, #41924, #58646, #32768)
@@ -1145,13 +1157,14 @@ nsHTMLCopyEncoder::SetSelection(nsISelection* aSelection)
   
   // also consider ourselves in a text widget if we can't find an html document
   nsCOMPtr<nsIHTMLDocument> htmlDoc = do_QueryInterface(mDocument);
-  if (!htmlDoc) mIsTextWidget = PR_TRUE;
+  if (!htmlDoc || mDocument->IsCaseSensitive())
+    mIsTextWidget = PR_TRUE;
   
   // normalize selection if we are not in a widget
   if (mIsTextWidget) 
   {
     mSelection = aSelection;
-    mMimeType = NS_LITERAL_STRING("text/plain");
+    mMimeType.AssignLiteral("text/plain");
     return NS_OK;
   }
   
@@ -1452,11 +1465,8 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
           nsCOMPtr<nsIContent> content = do_QueryInterface(parent);
           if (content)
           {
-            PRInt32 id;
-            parserService->HTMLAtomTagToId(content->Tag(), &id);
-
             PRBool isBlock = PR_FALSE;
-            parserService->IsBlock(id, isBlock);
+            parserService->IsBlock(parserService->HTMLAtomTagToId(content->Tag()), isBlock);
             if (isBlock)
             {
               bResetPromotion = PR_FALSE;
@@ -1538,11 +1548,8 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
           nsCOMPtr<nsIContent> content = do_QueryInterface(parent);
           if (content)
           {
-            PRInt32 id;
-            parserService->HTMLAtomTagToId(content->Tag(), &id);
-
             PRBool isBlock = PR_FALSE;
-            parserService->IsBlock(id, isBlock);
+            parserService->IsBlock(parserService->HTMLAtomTagToId(content->Tag()), isBlock);
             if (isBlock)
             {
               bResetPromotion = PR_FALSE;
@@ -1607,7 +1614,7 @@ nsHTMLCopyEncoder::IsMozBR(nsIDOMNode* aNode)
       nsAutoString typeAttrVal;
       nsresult rv = elem->GetAttribute(typeAttrName, typeAttrVal);
       ToLowerCase(typeAttrVal);
-      if (NS_SUCCEEDED(rv) && (typeAttrVal.Equals(NS_LITERAL_STRING("_moz"))))
+      if (NS_SUCCEEDED(rv) && (typeAttrVal.EqualsLiteral("_moz")))
         return PR_TRUE;
     }
     return PR_FALSE;
@@ -1743,7 +1750,7 @@ nsHTMLCopyEncoder::IsEmptyTextContent(nsIDOMNode* aNode)
   PRBool result = PR_FALSE;
   nsCOMPtr<nsITextContent> tc(do_QueryInterface(aNode));
   if (tc) {
-    tc->IsOnlyWhitespace(&result);
+    result = tc->IsOnlyWhitespace();
   }
   return result;
 }
