@@ -1,29 +1,46 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/*
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- * 
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
- * 
+// vim: ft=cpp tw=78 sw=2 et ts=2
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ *
  * The Original Code is Mozilla.
- * 
- * The Initial Developer of the Original Code is Netscape
- * Communications.  Portions created by Netscape Communications are
- * Copyright (C) 2001 by Netscape Communications.  All
- * Rights Reserved.
- * 
- * Contributor(s): 
+ *
+ * The Initial Developer of the Original Code is
+ * Netscape Communications.
+ * Portions created by the Initial Developer are Copyright (C) 2001
+ * the Initial Developer. All Rights Reserved.
+ *
+ * Contributor(s):
  *   Vidur Apparao <vidur@netscape.com> (original author)
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either of the GNU General Public License Version 2 or later (the "GPL"),
+ * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #include "nsScriptLoader.h"
 #include "nsIDOMCharacterData.h"
 #include "nsParserUtils.h"
+#include "nsIMIMEHeaderParam.h"
 #include "nsICharsetConverterManager.h"
 #include "nsIUnicodeDecoder.h"
 #include "nsIContent.h"
@@ -38,6 +55,7 @@
 #include "nsIDOMWindow.h"
 #include "nsIHttpChannel.h"
 #include "nsIScriptElement.h"
+#include "nsIDOMHTMLScriptElement.h"
 #include "nsIDocShell.h"
 #include "jsapi.h"
 #include "nsContentUtils.h"
@@ -88,34 +106,38 @@ IntersectPrincipalCerts(nsIPrincipal *aOld, nsIPrincipal *aNew)
 
 class nsScriptLoadRequest : public nsISupports {
 public:
-  nsScriptLoadRequest(nsIDOMHTMLScriptElement* aElement,
+  nsScriptLoadRequest(nsIScriptElement* aElement,
                       nsIScriptLoaderObserver* aObserver,
-                      const char* aVersionString);
+                      const char* aVersionString,
+                      PRBool aHasE4XOption);
   virtual ~nsScriptLoadRequest();
 
   NS_DECL_ISUPPORTS
-  
+
   void FireScriptAvailable(nsresult aResult,
                            const nsAFlatString& aScript);
   void FireScriptEvaluated(nsresult aResult);
-  
-  nsCOMPtr<nsIDOMHTMLScriptElement> mElement;
+
+  nsCOMPtr<nsIScriptElement> mElement;
   nsCOMPtr<nsIScriptLoaderObserver> mObserver;
   PRPackedBool mLoading;             // Are we still waiting for a load to complete?
   PRPackedBool mWasPending;          // Processed immediately or pending
   PRPackedBool mIsInline;            // Is the script inline or loaded?
+  PRPackedBool mHasE4XOption;        // Has E4X=1 script type parameter
   nsString mScriptText;              // Holds script for loaded scripts
   const char* mJSVersion;            // We don't own this string
   nsCOMPtr<nsIURI> mURI;
   PRInt32 mLineNo;
 };
 
-nsScriptLoadRequest::nsScriptLoadRequest(nsIDOMHTMLScriptElement* aElement,
+nsScriptLoadRequest::nsScriptLoadRequest(nsIScriptElement* aElement,
                                          nsIScriptLoaderObserver* aObserver,
-                                         const char* aVersionString) :
-  mElement(aElement), mObserver(aObserver), 
+                                         const char* aVersionString,
+                                         PRBool aHasE4XOption) :
+  mElement(aElement), mObserver(aObserver),
   mLoading(PR_TRUE), mWasPending(PR_FALSE),
-  mIsInline(PR_TRUE), mJSVersion(aVersionString), mLineNo(1)
+  mIsInline(PR_TRUE), mHasE4XOption(aHasE4XOption),
+  mJSVersion(aVersionString), mLineNo(1)
 {
 }
 
@@ -153,7 +175,7 @@ nsScriptLoadRequest::FireScriptEvaluated(nsresult aResult)
 //
 //////////////////////////////////////////////////////////////
 
-nsScriptLoader::nsScriptLoader() 
+nsScriptLoader::nsScriptLoader()
   : mDocument(nsnull), mEnabled(PR_TRUE)
 {
 }
@@ -161,7 +183,7 @@ nsScriptLoader::nsScriptLoader()
 nsScriptLoader::~nsScriptLoader()
 {
   mObservers.Clear();
-  
+
   PRInt32 count = mPendingRequests.Count();
   for (PRInt32 i = 0; i < count; i++) {
     nsScriptLoadRequest* req = mPendingRequests[i];
@@ -183,7 +205,7 @@ NS_IMPL_ADDREF(nsScriptLoader)
 NS_IMPL_RELEASE(nsScriptLoader)
 
 /* void init (in nsIDocument aDocument); */
-NS_IMETHODIMP 
+NS_IMETHODIMP
 nsScriptLoader::Init(nsIDocument *aDocument)
 {
   mDocument = aDocument;
@@ -192,7 +214,7 @@ nsScriptLoader::Init(nsIDocument *aDocument)
 }
 
 /* void dropDocumentReference (); */
-NS_IMETHODIMP 
+NS_IMETHODIMP
 nsScriptLoader::DropDocumentReference()
 {
   mDocument = nsnull;
@@ -201,35 +223,35 @@ nsScriptLoader::DropDocumentReference()
 }
 
 /* void addObserver (in nsIScriptLoaderObserver aObserver); */
-NS_IMETHODIMP 
+NS_IMETHODIMP
 nsScriptLoader::AddObserver(nsIScriptLoaderObserver *aObserver)
 {
   NS_ENSURE_ARG(aObserver);
-  
+
   mObservers.AppendObject(aObserver);
-  
+
   return NS_OK;
 }
 
 /* void removeObserver (in nsIScriptLoaderObserver aObserver); */
-NS_IMETHODIMP 
+NS_IMETHODIMP
 nsScriptLoader::RemoveObserver(nsIScriptLoaderObserver *aObserver)
 {
   NS_ENSURE_ARG(aObserver);
-  
+
   mObservers.RemoveObject(aObserver);
 
   return NS_OK;
 }
 
 PRBool
-nsScriptLoader::InNonScriptingContainer(nsIDOMHTMLScriptElement* aScriptElement)
+nsScriptLoader::InNonScriptingContainer(nsIScriptElement* aScriptElement)
 {
   nsCOMPtr<nsIDOMNode> node(do_QueryInterface(aScriptElement));
   nsCOMPtr<nsIDOMNode> parent;
 
   node->GetParentNode(getter_AddRefs(parent));
-  while (parent) {   
+  while (parent) {
     nsCOMPtr<nsIContent> content(do_QueryInterface(parent));
     if (!content) {
       break;
@@ -240,7 +262,7 @@ nsScriptLoader::InNonScriptingContainer(nsIDOMHTMLScriptElement* aScriptElement)
 
     if (nodeInfo) {
       nsIAtom *localName = nodeInfo->NameAtom();
-      
+
       // XXX noframes and noembed are currently unconditionally not
       // displayed and processed. This might change if we support either
       // prefs or per-document container settings for not allowing
@@ -269,7 +291,7 @@ nsScriptLoader::InNonScriptingContainer(nsIDOMHTMLScriptElement* aScriptElement)
 // <script for=... event=...> element.
 
 PRBool
-nsScriptLoader::IsScriptEventHandler(nsIDOMHTMLScriptElement *aScriptElement)
+nsScriptLoader::IsScriptEventHandler(nsIScriptElement *aScriptElement)
 {
   nsCOMPtr<nsIContent> contElement = do_QueryInterface(aScriptElement);
   if (!contElement ||
@@ -285,8 +307,7 @@ nsScriptLoader::IsScriptEventHandler(nsIDOMHTMLScriptElement *aScriptElement)
 
   const nsAString& for_str = nsContentUtils::TrimWhitespace(str);
 
-  if (!for_str.Equals(NS_LITERAL_STRING("window"),
-                      nsCaseInsensitiveStringComparator())) {
+  if (!for_str.LowerCaseEqualsLiteral("window")) {
     return PR_TRUE;
   }
 
@@ -326,9 +347,9 @@ nsScriptLoader::IsScriptEventHandler(nsIDOMHTMLScriptElement *aScriptElement)
   return PR_FALSE;
 }
 
-/* void processScriptElement (in nsIDOMHTMLScriptElement aElement, in nsIScriptLoaderObserver aObserver); */
-NS_IMETHODIMP 
-nsScriptLoader::ProcessScriptElement(nsIDOMHTMLScriptElement *aElement, 
+/* void processScriptElement (in nsIScriptElement aElement, in nsIScriptLoaderObserver aObserver); */
+NS_IMETHODIMP
+nsScriptLoader::ProcessScriptElement(nsIScriptElement *aElement,
                                      nsIScriptLoaderObserver *aObserver)
 {
   NS_ENSURE_ARG(aElement);
@@ -356,7 +377,7 @@ nsScriptLoader::ProcessScriptElement(nsIDOMHTMLScriptElement *aElement,
   if (!mDocument->IsScriptEnabled()) {
     return FireErrorNotification(NS_ERROR_NOT_AVAILABLE, aElement, aObserver);
   }
-  
+
   // Script evaluation can also be disabled in the current script
   // context even though it's enabled in the document.
   nsIScriptGlobalObject *globalObject = mDocument->GetScriptGlobalObject();
@@ -373,82 +394,133 @@ nsScriptLoader::ProcessScriptElement(nsIDOMHTMLScriptElement *aElement,
   }
 
   PRBool isJavaScript = PR_TRUE;
+  PRBool hasE4XOption = PR_FALSE;
   const char* jsVersionString = nsnull;
   nsAutoString language, type, src;
 
-  // Check the language attribute first, so type can trump language.
-  aElement->GetAttribute(NS_LITERAL_STRING("language"), language);
-  if (!language.IsEmpty()) {
-    isJavaScript = nsParserUtils::IsJavaScriptLanguage(language, &jsVersionString);
+  // "language" is a deprecated attribute of HTML, so we check it only for
+  // HTML script elements.
+  nsCOMPtr<nsIDOMHTMLScriptElement> htmlScriptElement =
+    do_QueryInterface(aElement);
+  if (htmlScriptElement) {
+    // Check the language attribute first, so type can trump language.
+    htmlScriptElement->GetAttribute(NS_LITERAL_STRING("language"), language);
+    if (!language.IsEmpty()) {
+      isJavaScript = nsParserUtils::IsJavaScriptLanguage(language,
+                                                         &jsVersionString);
+
+      // IE, Opera, etc. do not respect language version, so neither should
+      // we at this late date in the browser wars saga.  Note that this change
+      // affects HTML but not XUL or SVG (but note also that XUL has its own
+      // code to check nsParserUtils::IsJavaScriptLanguage -- that's probably
+      // a separate bug, one we may not be able to fix short of XUL2).  See
+      // bug 255895 (https://bugzilla.mozilla.org/show_bug.cgi?id=255895).
+      jsVersionString = ::JS_VersionToString(JSVERSION_DEFAULT);
+    }
   }
 
   // Check the type attribute to determine language and version.
-  aElement->GetType(type);
+  aElement->GetScriptType(type);
   if (!type.IsEmpty()) {
-    nsAutoString  mimeType;
-    nsAutoString  params;
-    nsParserUtils::SplitMimeType(type, mimeType, params);
+    nsCOMPtr<nsIMIMEHeaderParam> mimeHdrParser =
+      do_GetService("@mozilla.org/network/mime-hdrparam;1");
+    NS_ENSURE_TRUE(mimeHdrParser, NS_ERROR_FAILURE);
+
+    NS_ConvertUTF16toUTF8 typeAndParams(type);
+
+    nsAutoString mimeType;
+    rv = mimeHdrParser->GetParameter(typeAndParams, nsnull,
+                                     EmptyCString(), PR_FALSE, nsnull,
+                                     mimeType);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // Table ordered from most to least likely JS MIME types.
+    // See bug 62485, feel free to add <script type="..."> survey data to it,
+    // or to a new bug once 62485 is closed.
+    static const char *jsTypes[] = {
+      "text/javascript",
+      "text/ecmascript",
+      "application/javascript",
+      "application/ecmascript",
+      "application/x-javascript",
+      nsnull
+    };
+
+    isJavaScript = PR_FALSE;
+    for (PRInt32 i = 0; jsTypes[i]; i++) {
+      if (mimeType.LowerCaseEqualsASCII(jsTypes[i])) {
+        isJavaScript = PR_TRUE;
+        break;
+      }
+    }
 
-    isJavaScript = mimeType.EqualsIgnoreCase("application/x-javascript") || 
-                   mimeType.EqualsIgnoreCase("text/javascript");
     if (isJavaScript) {
       JSVersion jsVersion = JSVERSION_DEFAULT;
-      if (params.Find("version=", PR_TRUE) == 0) {
-        if (params.Length() != 11 || params[8] != '1' || params[9] != '.')
+      nsAutoString value;
+      rv = mimeHdrParser->GetParameter(typeAndParams, "version",
+                                       EmptyCString(), PR_FALSE, nsnull,
+                                       value);
+      if (NS_FAILED(rv)) {
+        if (rv != NS_ERROR_INVALID_ARG)
+          return rv;
+      } else {
+        if (value.Length() != 3 || value[0] != '1' || value[1] != '.')
           jsVersion = JSVERSION_UNKNOWN;
-        else switch (params[10]) {
-        case '0': jsVersion = JSVERSION_1_0; break;
-        case '1': jsVersion = JSVERSION_1_1; break;
-        case '2': jsVersion = JSVERSION_1_2; break;
-        case '3': jsVersion = JSVERSION_1_3; break;
-        case '4': jsVersion = JSVERSION_1_4; break;
-        case '5': jsVersion = JSVERSION_1_5; break;
-        default:  jsVersion = JSVERSION_UNKNOWN;
+        else switch (value[2]) {
+          case '0': jsVersion = JSVERSION_1_0; break;
+          case '1': jsVersion = JSVERSION_1_1; break;
+          case '2': jsVersion = JSVERSION_1_2; break;
+          case '3': jsVersion = JSVERSION_1_3; break;
+          case '4': jsVersion = JSVERSION_1_4; break;
+          case '5': jsVersion = JSVERSION_1_5; break;
+          case '6': jsVersion = JSVERSION_1_6; break;
+          default:  jsVersion = JSVERSION_UNKNOWN;
         }
       }
-      jsVersionString = JS_VersionToString(jsVersion);
+      jsVersionString = ::JS_VersionToString(jsVersion);
+
+      rv = mimeHdrParser->GetParameter(typeAndParams, "e4x",
+                                       EmptyCString(), PR_FALSE, nsnull,
+                                       value);
+      if (NS_FAILED(rv)) {
+        if (rv != NS_ERROR_INVALID_ARG)
+          return rv;
+      } else {
+        if (value.Length() == 1 && value[0] == '1')
+          hasE4XOption = PR_TRUE;
+      }
     }
   }
 
   // If this isn't JavaScript, we don't know how to evaluate.
-  // XXX How and where should we deal with other scripting languages??
+  // XXX How and where should we deal with other scripting languages?
+  //     See bug 255942 (https://bugzilla.mozilla.org/show_bug.cgi?id=255942).
   if (!isJavaScript) {
     return FireErrorNotification(NS_ERROR_NOT_AVAILABLE, aElement, aObserver);
   }
 
   // Create a request object for this script
-  nsRefPtr<nsScriptLoadRequest> request = new nsScriptLoadRequest(aElement, aObserver, jsVersionString);
+  nsRefPtr<nsScriptLoadRequest> request = new nsScriptLoadRequest(aElement, aObserver, jsVersionString, hasE4XOption);
   if (!request) {
     return FireErrorNotification(NS_ERROR_OUT_OF_MEMORY, aElement, aObserver);
   }
 
-  // Check to see if we have a src attribute.
-  aElement->GetSrc(src);
-  if (!src.IsEmpty()) {
-    nsCOMPtr<nsIURI> scriptURI;
-
-    // Use the SRC attribute value to load the URL
-    nsCOMPtr<nsIContent> content(do_QueryInterface(aElement));
-    NS_ASSERTION(content, "nsIDOMHTMLScriptElement not implementing nsIContent");
-    nsCOMPtr<nsIURI> baseURI = content->GetBaseURI();
-    rv = NS_NewURI(getter_AddRefs(scriptURI), src, nsnull, baseURI);
-
-    if (NS_FAILED(rv)) {
-      return FireErrorNotification(rv, aElement, aObserver);
-    }
-
+  // First check to see if this is an external script
+  nsCOMPtr<nsIURI> scriptURI = aElement->GetScriptURI();
+  if (scriptURI) {
     // Check that the containing page is allowed to load this URI.
-    nsIURI *docURI = mDocument->GetDocumentURI();
-    if (!docURI) {
+    nsIPrincipal *docPrincipal = mDocument->GetPrincipal();
+    if (!docPrincipal) {
       return FireErrorNotification(NS_ERROR_UNEXPECTED, aElement, aObserver);
     }
     rv = nsContentUtils::GetSecurityManager()->
-      CheckLoadURI(docURI, scriptURI, nsIScriptSecurityManager::ALLOW_CHROME);
+      CheckLoadURIWithPrincipal(docPrincipal, scriptURI,
+                                nsIScriptSecurityManager::ALLOW_CHROME);
 
     if (NS_FAILED(rv)) {
       return FireErrorNotification(rv, aElement, aObserver);
     }
-    
+
     // After the security manager, the content-policy stuff gets a veto
     if (globalObject) {
       PRInt16 shouldLoad = nsIContentPolicy::ACCEPT;
@@ -509,12 +581,7 @@ nsScriptLoader::ProcessScriptElement(nsIDOMHTMLScriptElement *aElement,
     request->mIsInline = PR_TRUE;
     request->mURI = mDocument->GetDocumentURI();
 
-    nsCOMPtr<nsIScriptElement> scriptElement(do_QueryInterface(aElement));
-    if (scriptElement) {
-      PRUint32 lineNumber;
-      scriptElement->GetLineNumber(&lineNumber);
-      request->mLineNo = lineNumber;
-    }
+    request->mLineNo = aElement->GetScriptLineNumber();
 
     // If we've got existing pending requests, add ourselves
     // to this list.
@@ -527,13 +594,24 @@ nsScriptLoader::ProcessScriptElement(nsIDOMHTMLScriptElement *aElement,
       rv = ProcessRequest(request);
     }
   }
-  
+
   return rv;
 }
 
+nsresult
+nsScriptLoader::GetCurrentScript(nsIScriptElement **aElement)
+{
+  NS_ENSURE_ARG_POINTER(aElement);
+  *aElement = mCurrentScript;
+
+  NS_IF_ADDREF(*aElement);
+
+  return NS_OK;
+}
+
 nsresult
 nsScriptLoader::FireErrorNotification(nsresult aResult,
-                                      nsIDOMHTMLScriptElement* aElement,
+                                      nsIScriptElement* aElement,
                                       nsIScriptLoaderObserver* aObserver)
 {
   PRInt32 count = mObservers.Count();
@@ -541,7 +619,7 @@ nsScriptLoader::FireErrorNotification(nsresult aResult,
     nsCOMPtr<nsIScriptLoaderObserver> observer = mObservers[i];
 
     if (observer) {
-      observer->ScriptAvailable(aResult, aElement, 
+      observer->ScriptAvailable(aResult, aElement,
                                 PR_TRUE, PR_FALSE,
                                 nsnull, 0,
                                 EmptyString());
@@ -549,7 +627,7 @@ nsScriptLoader::FireErrorNotification(nsresult aResult,
   }
 
   if (aObserver) {
-    aObserver->ScriptAvailable(aResult, aElement, 
+    aObserver->ScriptAvailable(aResult, aElement,
                                PR_TRUE, PR_FALSE,
                                nsnull, 0,
                                EmptyString());
@@ -569,7 +647,7 @@ nsScriptLoader::ProcessRequest(nsScriptLoadRequest* aRequest)
   if (aRequest->mIsInline) {
     // XXX This is inefficient - GetText makes multiple
     // copies.
-    aRequest->mElement->GetText(textData);
+    aRequest->mElement->GetScriptText(textData);
 
     script = &textData;
   }
@@ -594,7 +672,7 @@ nsScriptLoader::FireScriptAvailable(nsresult aResult,
     nsCOMPtr<nsIScriptLoaderObserver> observer = mObservers[i];
 
     if (observer) {
-      observer->ScriptAvailable(aResult, aRequest->mElement, 
+      observer->ScriptAvailable(aResult, aRequest->mElement,
                                 aRequest->mIsInline, aRequest->mWasPending,
                                 aRequest->mURI, aRequest->mLineNo,
                                 aScript);
@@ -618,7 +696,7 @@ nsScriptLoader::FireScriptEvaluated(nsresult aResult,
     }
   }
 
-  aRequest->FireScriptEvaluated(aResult);  
+  aRequest->FireScriptEvaluated(aResult);
 }
 
 nsresult
@@ -648,7 +726,6 @@ nsScriptLoader::EvaluateScript(nsScriptLoadRequest* aRequest,
   // have one.
   NS_ASSERTION(principal, "principal required for document");
 
-  nsAutoString ret;
   nsCAutoString url;
 
   if (aRequest->mURI) {
@@ -658,14 +735,45 @@ nsScriptLoader::EvaluateScript(nsScriptLoadRequest* aRequest,
     }
   }
 
+  PRBool oldProcessingScriptTag = context->GetProcessingScriptTag();
   context->SetProcessingScriptTag(PR_TRUE);
 
-  PRBool isUndefined;
-  context->EvaluateString(aScript, nsnull, principal, url.get(),
-                          aRequest->mLineNo, aRequest->mJSVersion, 
-                          ret, &isUndefined);  
+  JSContext *cx = (JSContext *)context->GetNativeContext();
+  uint32 options = ::JS_GetOptions(cx);
+  JSBool changed = (aRequest->mHasE4XOption ^ !!(options & JSOPTION_XML));
+  if (changed) {
+    ::JS_SetOptions(cx,
+                    aRequest->mHasE4XOption
+                    ? options | JSOPTION_XML
+                    : options & ~JSOPTION_XML);
+  }
 
-  context->SetProcessingScriptTag(PR_FALSE);
+  // Update our current script.
+  nsCOMPtr<nsIScriptElement> oldCurrent = mCurrentScript;
+  mCurrentScript = aRequest->mElement;
+
+  PRBool isUndefined;
+  context->EvaluateString(aScript, globalObject->GetGlobalJSObject(),
+                          principal, url.get(), aRequest->mLineNo,
+                          aRequest->mJSVersion, nsnull, &isUndefined);
+
+  // Put the old script back in case it wants to do anything else.
+  mCurrentScript = oldCurrent;
+
+  ::JS_ReportPendingException(cx);
+  if (changed) {
+    ::JS_SetOptions(cx, options);
+  }
+
+  context->SetProcessingScriptTag(oldProcessingScriptTag);
+
+  nsCOMPtr<nsIXPCNativeCallContext> ncc;
+  nsContentUtils::XPConnect()->
+    GetCurrentNativeCallContext(getter_AddRefs(ncc));
+
+  if (ncc) {
+    ncc->SetExceptionWasThrown(PR_FALSE);
+  }
 
   return rv;
 }
@@ -676,7 +784,7 @@ nsScriptLoader::ProcessPendingReqests()
   if (mPendingRequests.Count() == 0) {
     return;
   }
-  
+
   nsRefPtr<nsScriptLoadRequest> request = mPendingRequests[0];
   while (request && !request->mLoading) {
     mPendingRequests.RemoveObjectAt(0);
@@ -689,46 +797,134 @@ nsScriptLoader::ProcessPendingReqests()
 }
 
 
-// This function is copied from nsParser.cpp. It was simplified though, unnecessary part is removed.
-static PRBool 
-DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsCString& oCharset) 
+// This function was copied from nsParser.cpp. It was simplified a bit.
+static PRBool
+DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsCString& oCharset)
 {
   if (aLen < 2)
     return PR_FALSE;
 
   switch(aBytes[0]) {
-  case 0xEF:  
-    if( aLen >= 3 && (0xBB==aBytes[1]) && (0xBF==aBytes[2])) {
-        // EF BB BF
-        // Win2K UTF-8 BOM
-        oCharset.Assign("UTF-8"); 
+  case 0xEF:
+    if (aLen >= 3 && 0xBB == aBytes[1] && 0xBF == aBytes[2]) {
+      // EF BB BF
+      // Win2K UTF-8 BOM
+      oCharset.Assign("UTF-8");
     }
     break;
   case 0xFE:
-    if(0xFF==aBytes[1]) {
+    if (0xFF == aBytes[1]) {
       // FE FF
-      // UTF-16, big-endian 
-      oCharset.Assign("UTF-16BE"); 
+      // UTF-16, big-endian
+      oCharset.Assign("UTF-16BE");
     }
     break;
   case 0xFF:
-    if(0xFE==aBytes[1]) {
+    if (0xFE == aBytes[1]) {
       // FF FE
-      // UTF-16, little-endian 
-      oCharset.Assign("UTF-16LE"); 
+      // UTF-16, little-endian
+      oCharset.Assign("UTF-16LE");
     }
     break;
-  }  // switch
+  }
   return !oCharset.IsEmpty();
 }
 
+/* static */ nsresult
+nsScriptLoader::ConvertToUTF16(nsIChannel* aChannel, const PRUint8* aData,
+                               PRUint32 aLength, const nsString& aHintCharset,
+                               nsIDocument* aDocument, nsString& aString)
+{
+  if (!aLength) {
+    aString.Truncate();
+    return NS_OK;
+  }
+
+  nsCAutoString characterSet;
+
+  nsresult rv = NS_OK;
+  if (aChannel) {
+    rv = aChannel->GetContentCharset(characterSet);
+  }
+
+  if (!aHintCharset.IsEmpty() && (NS_FAILED(rv) || characterSet.IsEmpty())) {
+    // charset name is always ASCII.
+    LossyCopyUTF16toASCII(aHintCharset, characterSet);
+  }
+
+  if (NS_FAILED(rv) || characterSet.IsEmpty()) {
+    DetectByteOrderMark(aData, aLength, characterSet);
+  }
+
+  if (characterSet.IsEmpty()) {
+    // charset from document default
+    characterSet = aDocument->GetDocumentCharacterSet();
+  }
+
+  if (characterSet.IsEmpty()) {
+    // fall back to ISO-8859-1, see bug 118404
+    characterSet.AssignLiteral("ISO-8859-1");
+  }
+
+  nsCOMPtr<nsICharsetConverterManager> charsetConv =
+    do_GetService(kCharsetConverterManagerCID, &rv);
+
+  nsCOMPtr<nsIUnicodeDecoder> unicodeDecoder;
+
+  if (NS_SUCCEEDED(rv) && charsetConv) {
+    rv = charsetConv->GetUnicodeDecoder(characterSet.get(),
+                                        getter_AddRefs(unicodeDecoder));
+    if (NS_FAILED(rv)) {
+      // fall back to ISO-8859-1 if charset is not supported. (bug 230104)
+      rv = charsetConv->GetUnicodeDecoderRaw("ISO-8859-1",
+                                             getter_AddRefs(unicodeDecoder));
+    }
+  }
+
+  // converts from the charset to unicode
+  if (NS_SUCCEEDED(rv)) {
+    PRInt32 unicodeLength = 0;
+
+    rv = unicodeDecoder->GetMaxLength(NS_REINTERPRET_CAST(const char*, aData),
+                                      aLength, &unicodeLength);
+    if (NS_SUCCEEDED(rv)) {
+      aString.SetLength(unicodeLength);
+      PRUnichar *ustr = aString.BeginWriting();
+
+      PRInt32 consumedLength = 0;
+      PRInt32 originalLength = aLength;
+      PRInt32 convertedLength = 0;
+      PRInt32 bufferLength = unicodeLength;
+      do {
+        rv = unicodeDecoder->Convert(NS_REINTERPRET_CAST(const char*, aData),
+                                     (PRInt32 *) &aLength, ustr,
+                                     &unicodeLength);
+        if (NS_FAILED(rv)) {
+          // if we failed, we consume one byte, replace it with U+FFFD
+          // and try the conversion again.
+          ustr[unicodeLength++] = (PRUnichar)0xFFFD;
+          ustr += unicodeLength;
+
+          unicodeDecoder->Reset();
+        }
+        aData += ++aLength;
+        consumedLength += aLength;
+        aLength = originalLength - consumedLength;
+        convertedLength += unicodeLength;
+        unicodeLength = bufferLength - convertedLength;
+      } while (NS_FAILED(rv) && (originalLength > consumedLength) && (bufferLength > convertedLength));
+      aString.SetLength(convertedLength);
+    }
+  }
+  return rv;
+}
 
 NS_IMETHODIMP
 nsScriptLoader::OnStreamComplete(nsIStreamLoader* aLoader,
                                  nsISupports* aContext,
                                  nsresult aStatus,
                                  PRUint32 stringLen,
-                                 const char* string)
+                                 const PRUint8* string)
 {
   nsresult rv;
   nsScriptLoadRequest* request = NS_STATIC_CAST(nsScriptLoadRequest*, aContext);
@@ -748,7 +944,7 @@ nsScriptLoader::OnStreamComplete(nsIStreamLoader* aLoader,
   // evaluation.
   if (!mDocument) {
     mPendingRequests.RemoveObject(request);
-    FireScriptAvailable(NS_ERROR_NOT_AVAILABLE, request, 
+    FireScriptAvailable(NS_ERROR_NOT_AVAILABLE, request,
                         EmptyString());
     ProcessPendingReqests();
     return NS_OK;
@@ -756,7 +952,7 @@ nsScriptLoader::OnStreamComplete(nsIStreamLoader* aLoader,
 
   // If the load returned an error page, then we need to abort
   nsCOMPtr<nsIRequest> req;
-  rv = aLoader->GetRequest(getter_AddRefs(req));    
+  rv = aLoader->GetRequest(getter_AddRefs(req));
   NS_ASSERTION(req, "StreamLoader's request went away prematurely");
   if (NS_FAILED(rv)) return rv;  // XXX Should this remove the pending request?
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(req));
@@ -765,97 +961,20 @@ nsScriptLoader::OnStreamComplete(nsIStreamLoader* aLoader,
     rv = httpChannel->GetRequestSucceeded(&requestSucceeded);
     if (NS_SUCCEEDED(rv) && !requestSucceeded) {
       mPendingRequests.RemoveObject(request);
-      FireScriptAvailable(NS_ERROR_NOT_AVAILABLE, request, 
+      FireScriptAvailable(NS_ERROR_NOT_AVAILABLE, request,
                           EmptyString());
       ProcessPendingReqests();
       return NS_OK;
     }
   }
 
+  nsCOMPtr<nsIChannel> channel = do_QueryInterface(req);
   if (stringLen) {
-    nsCAutoString characterSet;
-    nsCOMPtr<nsIChannel> channel;
-
-    channel = do_QueryInterface(req);
-    if (channel) {
-      rv = channel->GetContentCharset(characterSet);
-    }
-
-    if (NS_FAILED(rv) || characterSet.IsEmpty()) {
-      // Check the charset attribute to determine script charset.
-      nsAutoString charset;
-      rv = request->mElement->GetCharset(charset);
-      if (NS_SUCCEEDED(rv)) {  
-        // charset name is always ASCII.
-        LossyCopyUTF16toASCII(charset, characterSet);
-      }
-    }
-
-    if (NS_FAILED(rv) || characterSet.IsEmpty()) {
-      DetectByteOrderMark((const unsigned char*)string, stringLen, characterSet);
-    }
-
-    if (characterSet.IsEmpty()) {
-      // charset from document default
-      characterSet = mDocument->GetDocumentCharacterSet();
-    }
-
-    if (characterSet.IsEmpty()) {
-      // fall back to ISO-8859-1, see bug 118404
-      characterSet = NS_LITERAL_CSTRING("ISO-8859-1");
-    }
-    
-    nsCOMPtr<nsICharsetConverterManager> charsetConv =
-      do_GetService(kCharsetConverterManagerCID, &rv);
-
-    nsCOMPtr<nsIUnicodeDecoder> unicodeDecoder;
-
-    if (NS_SUCCEEDED(rv) && charsetConv) {
-      rv = charsetConv->GetUnicodeDecoder(characterSet.get(),
-                                          getter_AddRefs(unicodeDecoder));
-      if (NS_FAILED(rv)) {
-        // fall back to ISO-8859-1 if charset is not supported. (bug 230104)
-        rv = charsetConv->GetUnicodeDecoderRaw("ISO-8859-1",
-                                               getter_AddRefs(unicodeDecoder));
-      }
-    }
-
-    // converts from the charset to unicode
-    if (NS_SUCCEEDED(rv)) {
-      PRInt32 unicodeLength = 0;
-
-      rv = unicodeDecoder->GetMaxLength(string, stringLen, &unicodeLength);
-      if (NS_SUCCEEDED(rv)) {
-        nsString tempStr;
-        tempStr.SetLength(unicodeLength);
-        PRUnichar *ustr;
-        tempStr.BeginWriting(ustr);
-        
-        PRInt32 consumedLength = 0;
-        PRInt32 originalLength = stringLen;
-        PRInt32 convertedLength = 0;
-        PRInt32 bufferLength = unicodeLength;
-        do {
-          rv = unicodeDecoder->Convert(string, (PRInt32 *) &stringLen, ustr,
-                                       &unicodeLength);
-          if (NS_FAILED(rv)) {
-            // if we failed, we consume one byte, replace it with U+FFFD
-            // and try the conversion again.
-            ustr[unicodeLength++] = (PRUnichar)0xFFFD;
-            ustr += unicodeLength;
-
-            unicodeDecoder->Reset();
-          }
-          string += ++stringLen;
-          consumedLength += stringLen;
-          stringLen = originalLength - consumedLength;
-          convertedLength += unicodeLength;
-          unicodeLength = bufferLength - convertedLength;
-        } while (NS_FAILED(rv) && (originalLength > consumedLength) && (bufferLength > convertedLength));
-        tempStr.SetLength(convertedLength);
-        request->mScriptText = tempStr;
-      }
-    }
+    // Check the charset attribute to determine script charset.
+    nsAutoString hintCharset;
+    request->mElement->GetScriptCharset(hintCharset);
+    rv = ConvertToUTF16(channel, string, stringLen, hintCharset, mDocument,
+                        request->mScriptText);
 
     NS_ASSERTION(NS_SUCCEEDED(rv),
                  "Could not convert external JavaScript to Unicode!");
