@@ -1,11 +1,12 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim:set ts=4 sw=4 sts=4 et cindent: */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -14,7 +15,7 @@
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
@@ -24,16 +25,16 @@
  *   Andreas Otte <andreas.otte@debitel.net>
  *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
  * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -48,19 +49,23 @@
 #include "nsICharsetConverterManager.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
-#include "nsIPrefBranchInternal.h"
+#include "nsIPrefBranch2.h"
 #include "nsIIDNService.h"
+#include "nsIPlatformCharset.h"
 #include "nsNetUtil.h"
 #include "prlog.h"
+#include "nsAutoPtr.h"
 
 static NS_DEFINE_CID(kThisImplCID, NS_THIS_STANDARDURL_IMPL_CID);
 static NS_DEFINE_CID(kStandardURLCID, NS_STANDARDURL_CID);
 
-nsIIDNService *nsStandardURL::gIDNService = nsnull;
+nsIIDNService *nsStandardURL::gIDN = nsnull;
 nsICharsetConverterManager *nsStandardURL::gCharsetMgr = nsnull;
 PRBool nsStandardURL::gInitialized = PR_FALSE;
 PRBool nsStandardURL::gEscapeUTF8 = PR_TRUE;
-PRBool nsStandardURL::gAlwaysEncodeInUTF8 = PR_FALSE;
+PRBool nsStandardURL::gAlwaysEncodeInUTF8 = PR_TRUE;
+PRBool nsStandardURL::gShowPunycode = PR_FALSE;
+nsIPrefBranch *nsStandardURL::gIDNWhitelistPrefBranch = nsnull;
 
 #if defined(PR_LOGGING)
 //
@@ -133,6 +138,8 @@ end:
 #define NS_NET_PREF_ESCAPEUTF8         "network.standard-url.escape-utf8"
 #define NS_NET_PREF_ENABLEIDN          "network.enableIDN"
 #define NS_NET_PREF_ALWAYSENCODEINUTF8 "network.standard-url.encode-utf8"
+#define NS_NET_PREF_SHOWPUNYCODE       "network.IDN_show_punycode"
+#define NS_NET_PREF_IDNWHITELIST       "network.IDN.whitelist."
 
 NS_IMPL_ISUPPORTS1(nsStandardURL::nsPrefObserver, nsIObserver)
 
@@ -144,7 +151,7 @@ nsPrefObserver::Observe(nsISupports *subject,
     if (!strcmp(topic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID)) {
         nsCOMPtr<nsIPrefBranch> prefBranch( do_QueryInterface(subject) );
         if (prefBranch) {
-            PrefsChanged(prefBranch, NS_ConvertUCS2toUTF8(data).get());
+            PrefsChanged(prefBranch, NS_ConvertUTF16toUTF8(data).get());
         } 
     }
     return NS_OK;
@@ -180,7 +187,7 @@ nsSegmentEncoder::EncodeSegmentCount(const char *str,
         // null or the empty string then the origin charset is UTF-8 and there
         // is nothing to do.
         nsCAutoString encBuf;
-        if (!nsCRT::IsAscii(str + pos, len) && mCharset && *mCharset) {
+        if (mCharset && *mCharset && !nsCRT::IsAscii(str + pos, len)) {
             // we have to encode this segment
             if (mEncoder || InitUnicodeEncoder()) {
                 NS_ConvertUTF8toUCS2 ucsBuf(Substring(str + pos, str + pos + len));
@@ -250,7 +257,7 @@ nsSegmentEncoder::InitUnicodeEncoder()
 }
 
 #define GET_SEGMENT_ENCODER(name) \
-    nsSegmentEncoder name(mOriginCharset.get())
+    nsSegmentEncoder name(gAlwaysEncodeInUTF8 ? nsnull : mOriginCharset.get())
 
 //----------------------------------------------------------------------------
 // nsStandardURL <public>
@@ -259,10 +266,10 @@ nsSegmentEncoder::InitUnicodeEncoder()
 nsStandardURL::nsStandardURL(PRBool aSupportsFileURL)
     : mDefaultPort(-1)
     , mPort(-1)
-    , mURLType(URLTYPE_STANDARD)
     , mHostA(nsnull)
     , mHostEncoding(eEncoding_ASCII)
     , mSpecEncoding(eEncoding_Unknown)
+    , mURLType(URLTYPE_STANDARD)
     , mMutable(PR_TRUE)
     , mSupportsFileURL(aSupportsFileURL)
 {
@@ -292,22 +299,32 @@ nsStandardURL::~nsStandardURL()
 void
 nsStandardURL::InitGlobalObjects()
 {
-    nsCOMPtr<nsIPrefBranchInternal> prefBranch( do_GetService(NS_PREFSERVICE_CONTRACTID) );
+    nsCOMPtr<nsIPrefBranch2> prefBranch( do_GetService(NS_PREFSERVICE_CONTRACTID) );
     if (prefBranch) {
         nsCOMPtr<nsIObserver> obs( new nsPrefObserver() );
         prefBranch->AddObserver(NS_NET_PREF_ESCAPEUTF8, obs.get(), PR_FALSE); 
         prefBranch->AddObserver(NS_NET_PREF_ALWAYSENCODEINUTF8, obs.get(), PR_FALSE);
         prefBranch->AddObserver(NS_NET_PREF_ENABLEIDN, obs.get(), PR_FALSE); 
+        prefBranch->AddObserver(NS_NET_PREF_SHOWPUNYCODE, obs.get(), PR_FALSE); 
 
         PrefsChanged(prefBranch, nsnull);
+
+        nsCOMPtr<nsIPrefService> prefs = do_QueryInterface(prefBranch); 
+        if (prefs) {
+            nsCOMPtr<nsIPrefBranch> branch;
+           if (NS_SUCCEEDED(prefs->GetBranch( NS_NET_PREF_IDNWHITELIST,
+                                              getter_AddRefs(branch) )))
+               NS_ADDREF(gIDNWhitelistPrefBranch = branch);
+        }
     }
 }
 
 void
 nsStandardURL::ShutdownGlobalObjects()
 {
-    NS_IF_RELEASE(gIDNService);
+    NS_IF_RELEASE(gIDN);
     NS_IF_RELEASE(gCharsetMgr);
+    NS_IF_RELEASE(gIDNWhitelistPrefBranch);
 }
 
 //----------------------------------------------------------------------------
@@ -350,7 +367,7 @@ nsStandardURL::InvalidateCache(PRBool invalidateCachedFile)
 }
 
 PRBool
-nsStandardURL::EncodeHost(const char *host, nsCString &result)
+nsStandardURL::EscapeIPv6(const char *host, nsCString &result)
 {
     // Escape IPv6 address literal by surrounding it with []'s
     if (host && (host[0] != '[') && PL_strchr(host, ':')) {
@@ -362,6 +379,45 @@ nsStandardURL::EncodeHost(const char *host, nsCString &result)
     return PR_FALSE;
 }
 
+PRBool
+nsStandardURL::NormalizeIDN(const nsCSubstring &host, nsCString &result)
+{
+    // If host is ACE, then convert to UTF-8.  Else, if host is already UTF-8,
+    // then make sure it is normalized per IDN.
+
+    // this function returns PR_TRUE iff it writes something to |result|.
+
+    // NOTE: As a side-effect this function sets mHostEncoding.  While it would
+    // be nice to avoid side-effects in this function, the implementation of
+    // this function is already somewhat bound to the behavior of the
+    // callsites.  Anyways, this function exists to avoid code duplication, so
+    // side-effects abound :-/
+
+    NS_ASSERTION(mHostEncoding == eEncoding_ASCII, "unexpected default encoding");
+
+    if (IsASCII(host)) {
+        PRBool isACE;
+        if (gIDN &&
+            NS_SUCCEEDED(gIDN->IsACE(host, &isACE)) && isACE &&
+            NS_SUCCEEDED(ACEtoDisplayIDN(host, result))) {
+            mHostEncoding = eEncoding_UTF8;
+            return PR_TRUE;
+        }
+    }
+    else {
+        mHostEncoding = eEncoding_UTF8;
+        if (gIDN && NS_SUCCEEDED(UTF8toDisplayIDN(host, result))) {
+            // normalization could result in an ASCII only hostname
+            if (IsASCII(result))
+                mHostEncoding = eEncoding_ASCII;
+            return PR_TRUE;
+        }
+    }
+
+    result.Truncate();
+    return PR_FALSE;
+}
+
 void
 nsStandardURL::CoalescePath(netCoalesceFlags coalesceFlag, char *path)
 {
@@ -447,23 +503,12 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
     // However, perform Unicode normalization on it, as IDN does.
     mHostEncoding = eEncoding_ASCII;
     if (mHost.mLen > 0) {
-        useEncHost = PR_FALSE;
         const nsCSubstring& tempHost =
             Substring(spec + mHost.mPos, spec + mHost.mPos + mHost.mLen);
-        if (IsASCII(tempHost))
+        if ((useEncHost = NormalizeIDN(tempHost, encHost)))
+            approxLen += encHost.Length();
+        else
             approxLen += mHost.mLen;
-        else {
-            mHostEncoding = eEncoding_UTF8;
-            if (gIDNService &&
-                NS_SUCCEEDED(gIDNService->Normalize(tempHost, encHost))) {
-                approxLen += encHost.Length();
-                useEncHost = PR_TRUE;
-            }
-            else {
-                encHost.Truncate();
-                approxLen += mHost.mLen;
-            }
-        }
     }
 
     //
@@ -584,21 +629,6 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
     return NS_OK;
 }
 
-PRBool
-nsStandardURL::HostsAreEquivalent(nsStandardURL *that)
-{
-    // optimize for the non-IDN case...
-    if ((this->mHostEncoding == eEncoding_ASCII) &&
-        (that->mHostEncoding == eEncoding_ASCII))
-        return SegmentIs(mHost, that->mSpec.get(), that->mHost);
-
-    nsCAutoString thisHost, thatHost;
-    this->GetAsciiHost(thisHost);
-    that->GetAsciiHost(thatHost);
-    
-    return !nsCRT::strcasecmp(thisHost.get(), thatHost.get());
-}
-
 PRBool
 nsStandardURL::SegmentIs(const URLSegment &seg, const char *val)
 {
@@ -609,7 +639,7 @@ nsStandardURL::SegmentIs(const URLSegment &seg, const char *val)
         return PR_FALSE;
     // if the first |seg.mLen| chars of |val| match, then |val| must
     // also be null terminated at |seg.mLen|.
-    return !nsCRT::strncasecmp(mSpec.get() + seg.mPos, val, seg.mLen)
+    return !strncmp(mSpec.get() + seg.mPos, val, seg.mLen)
         && (val[seg.mLen] == '\0');
 }
 
@@ -623,7 +653,7 @@ nsStandardURL::SegmentIs(const char* spec, const URLSegment &seg, const char *va
         return PR_FALSE;
     // if the first |seg.mLen| chars of |val| match, then |val| must
     // also be null terminated at |seg.mLen|.
-    return !nsCRT::strncasecmp(spec + seg.mPos, val, seg.mLen)
+    return !strncmp(spec + seg.mPos, val, seg.mLen)
         && (val[seg.mLen] == '\0');
 }
 
@@ -634,7 +664,7 @@ nsStandardURL::SegmentIs(const URLSegment &seg1, const char *val, const URLSegme
         return PR_FALSE;
     if (seg1.mLen == -1 || (!val && mSpec.IsEmpty()))
         return PR_TRUE; // both are empty
-    return !nsCRT::strncasecmp(mSpec.get() + seg1.mPos, val + seg2.mPos, seg1.mLen); 
+    return !strncmp(mSpec.get() + seg1.mPos, val + seg2.mPos, seg1.mLen); 
 }
 
 PRInt32
@@ -664,14 +694,14 @@ nsStandardURL::ReplaceSegment(PRUint32 pos, PRUint32 len, const nsACString &val)
 }
 
 nsresult
-nsStandardURL::ParseURL(const char *spec)
+nsStandardURL::ParseURL(const char *spec, PRInt32 specLen)
 {
     nsresult rv;
 
     //
     // parse given URL string
     //
-    rv = mParser->ParseURL(spec, -1,
+    rv = mParser->ParseURL(spec, specLen,
                            &mScheme.mPos, &mScheme.mLen,
                            &mAuthority.mPos, &mAuthority.mLen,
                            &mPath.mPos, &mPath.mLen);
@@ -778,25 +808,25 @@ nsStandardURL::WriteSegment(nsIBinaryOutputStream *stream, const URLSegment &seg
     return NS_OK;
 }
 
-void
+/* static */ void
 nsStandardURL::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
 {
     PRBool val;
 
     LOG(("nsStandardURL::PrefsChanged [pref=%s]\n", pref));
 
-#define PREF_CHANGED(p) ((pref == nsnull) || !PL_strcmp(pref, p))
+#define PREF_CHANGED(p) ((pref == nsnull) || !strcmp(pref, p))
 #define GOT_PREF(p, b) (NS_SUCCEEDED(prefs->GetBoolPref(p, &b)))
 
     if (PREF_CHANGED(NS_NET_PREF_ENABLEIDN)) {
-        NS_IF_RELEASE(gIDNService);
+        NS_IF_RELEASE(gIDN);
         if (GOT_PREF(NS_NET_PREF_ENABLEIDN, val) && val) {
             // initialize IDN
             nsCOMPtr<nsIIDNService> serv(do_GetService(NS_IDNSERVICE_CONTRACTID));
             if (serv)
-                NS_ADDREF(gIDNService = serv.get());
+                NS_ADDREF(gIDN = serv.get());
         }
-        LOG(("IDN support %s\n", gIDNService ? "enabled" : "disabled"));
+        LOG(("IDN support %s\n", gIDN ? "enabled" : "disabled"));
     }
     
     if (PREF_CHANGED(NS_NET_PREF_ESCAPEUTF8)) {
@@ -810,9 +840,52 @@ nsStandardURL::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
             gAlwaysEncodeInUTF8 = val;
         LOG(("encode in UTF-8 %s\n", gAlwaysEncodeInUTF8 ? "enabled" : "disabled"));
     }
+
+    if (PREF_CHANGED(NS_NET_PREF_SHOWPUNYCODE)) {
+        if (GOT_PREF(NS_NET_PREF_SHOWPUNYCODE, val))
+            gShowPunycode = val;
+        LOG(("show punycode %s\n", gShowPunycode ? "enabled" : "disabled"));
+    }
 #undef PREF_CHANGED
 #undef GOT_PREF
 }
+
+/* static */ nsresult
+nsStandardURL::ACEtoDisplayIDN(const nsCSubstring &host, nsCString &result)
+{
+    if (gShowPunycode || !IsInWhitelist(host)) {
+        result = host;
+        return NS_OK;
+    }
+
+    return gIDN->ConvertACEtoUTF8(host, result);
+}
+
+/* static */ nsresult
+nsStandardURL::UTF8toDisplayIDN(const nsCSubstring &host, nsCString &result)
+{
+    if (gShowPunycode || !IsInWhitelist(host))
+        return gIDN->ConvertUTF8toACE(host, result);
+
+    return gIDN->Normalize(host, result);
+}
+
+/* static */ PRBool
+nsStandardURL::IsInWhitelist(const nsCSubstring &host)
+{
+    PRInt32 pos; 
+    PRBool safe;
+
+    if (gIDNWhitelistPrefBranch && 
+        (pos = nsCAutoString(host).RFind(".")) != kNotFound &&
+        NS_SUCCEEDED(gIDNWhitelistPrefBranch->
+                     GetBoolPref(nsCAutoString(Substring(host, pos + 1)).get(),
+                                 &safe)))
+        return safe;
+
+    return PR_FALSE;
+}
+
 //----------------------------------------------------------------------------
 // nsStandardURL::nsISupports
 //----------------------------------------------------------------------------
@@ -970,9 +1043,9 @@ nsStandardURL::GetAsciiHost(nsACString &result)
         return NS_OK;
     }
 
-    if (gIDNService) {
+    if (gIDN) {
         nsresult rv;
-        rv = gIDNService->ConvertUTF8toACE(Host(), result);
+        rv = gIDN->ConvertUTF8toACE(Host(), result);
         if (NS_SUCCEEDED(rv)) {
             mHostA = ToNewCString(result);
             return NS_OK;
@@ -989,7 +1062,7 @@ NS_IMETHODIMP
 nsStandardURL::GetOriginCharset(nsACString &result)
 {
     if (mOriginCharset.IsEmpty())
-        result = NS_LITERAL_CSTRING("UTF-8");
+        result.AssignLiteral("UTF-8");
     else
         result = mOriginCharset;
     return NS_OK;
@@ -1002,6 +1075,7 @@ nsStandardURL::SetSpec(const nsACString &input)
 
     const nsPromiseFlatCString &flat = PromiseFlatCString(input);
     const char *spec = flat.get();
+    PRInt32 specLength = flat.Length();
 
     LOG(("nsStandardURL::SetSpec [spec=%s]\n", spec));
 
@@ -1012,11 +1086,13 @@ nsStandardURL::SetSpec(const nsACString &input)
 
     // filter out unexpected chars "\r\n\t" if necessary
     nsCAutoString buf1;
-    if (net_FilterURIString(spec, buf1))
+    if (net_FilterURIString(spec, buf1)) {
         spec = buf1.get();
+        specLength = buf1.Length();
+    }
 
     // parse the given URL...
-    nsresult rv = ParseURL(spec);
+    nsresult rv = ParseURL(spec, specLength);
     if (NS_FAILED(rv)) return rv;
 
     // finally, use the URLSegment member variables to build a normalized
@@ -1095,6 +1171,8 @@ nsStandardURL::SetUserPass(const nsACString &input)
     LOG(("nsStandardURL::SetUserPass [userpass=%s]\n", userpass.get()));
 
     if (mURLType == URLTYPE_NO_AUTHORITY) {
+        if (userpass.IsEmpty())
+            return NS_OK;
         NS_ERROR("cannot set user:pass on no-auth url");
         return NS_ERROR_UNEXPECTED;
     }
@@ -1194,6 +1272,8 @@ nsStandardURL::SetUsername(const nsACString &input)
     LOG(("nsStandardURL::SetUsername [username=%s]\n", username.get()));
 
     if (mURLType == URLTYPE_NO_AUTHORITY) {
+        if (username.IsEmpty())
+            return NS_OK;
         NS_ERROR("cannot set username on no-auth url");
         return NS_ERROR_UNEXPECTED;
     }
@@ -1237,6 +1317,8 @@ nsStandardURL::SetPassword(const nsACString &input)
     LOG(("nsStandardURL::SetPassword [password=%s]\n", password.get()));
 
     if (mURLType == URLTYPE_NO_AUTHORITY) {
+        if (password.IsEmpty())
+            return NS_OK;
         NS_ERROR("cannot set password on no-auth url");
         return NS_ERROR_UNEXPECTED;
     }
@@ -1303,6 +1385,8 @@ nsStandardURL::SetHost(const nsACString &input)
     LOG(("nsStandardURL::SetHost [host=%s]\n", host));
 
     if (mURLType == URLTYPE_NO_AUTHORITY) {
+        if (flat.IsEmpty())
+            return NS_OK;
         NS_ERROR("cannot set host on no-auth url");
         return NS_ERROR_UNEXPECTED;
     }
@@ -1327,22 +1411,17 @@ nsStandardURL::SetHost(const nsACString &input)
 
     // handle IPv6 unescaped address literal
     PRInt32 len;
-    nsCAutoString escapedHost;
-    if (EncodeHost(host, escapedHost)) {
-        host = escapedHost.get();
-        len = escapedHost.Length();
+    nsCAutoString hostBuf;
+    if (EscapeIPv6(host, hostBuf)) {
+        host = hostBuf.get();
+        len = hostBuf.Length();
     }
-    else {
+    else if (NormalizeIDN(flat, hostBuf)) {
+        host = hostBuf.get();
+        len = hostBuf.Length();
+    }
+    else
         len = flat.Length();
-        if (!IsASCII(flat)) {
-            mHostEncoding = eEncoding_UTF8;
-            if (gIDNService &&
-                NS_SUCCEEDED(gIDNService->Normalize(flat, escapedHost))) {
-                host = escapedHost.get();
-                len = escapedHost.Length();
-            }
-        }
-    }
 
     if (mHost.mLen < 0) {
         mHost.mPos = mAuthority.mPos;
@@ -1356,6 +1435,10 @@ nsStandardURL::SetHost(const nsACString &input)
         mAuthority.mLen += shift;
         ShiftFromPath(shift);
     }
+
+    // Now canonicalize the host to lowercase
+    net_ToLowerCase(mSpec.BeginWriting() + mHost.mPos, mHost.mLen);
+
     return NS_OK;
 }
              
@@ -1369,6 +1452,11 @@ nsStandardURL::SetPort(PRInt32 port)
     if ((port == mPort) || (mPort == -1 && port == mDefaultPort))
         return NS_OK;
 
+    if (mURLType == URLTYPE_NO_AUTHORITY) {
+        NS_ERROR("cannot set port on no-auth url");
+        return NS_ERROR_UNEXPECTED;
+    }
+
     InvalidateCache();
 
     if (mPort == -1) {
@@ -1443,30 +1531,82 @@ nsStandardURL::Equals(nsIURI *unknownOther, PRBool *result)
     NS_ENSURE_ARG_POINTER(unknownOther);
     NS_PRECONDITION(result, "null pointer");
 
-    nsStandardURL *other;
-    nsresult rv = unknownOther->QueryInterface(kThisImplCID, (void **) &other);
+    nsRefPtr<nsStandardURL> otherPtr;
+    nsresult rv = unknownOther->QueryInterface(kThisImplCID,
+                                               getter_AddRefs(otherPtr));
+
+    // Hack around issue with MSVC++ not allowing the nsDerivedSafe to access
+    // the private members and not doing the implicit conversion to a raw
+    // pointer.
+    nsStandardURL* other = otherPtr;
     if (NS_FAILED(rv)) {
         *result = PR_FALSE;
         return NS_OK;
     }
 
-    // XXX this needs to allow for case insensitive paths in some cases (e.g.,
-    // file: URLs under windows).
+    // First, check whether one URIs is an nsIFileURL while the other
+    // is not.  If that's the case, they're different.
+    if (mSupportsFileURL != other->mSupportsFileURL) {
+        *result = PR_FALSE;
+        return NS_OK;
+    }
 
-    *result = 
-        SegmentIs(mScheme, other->mSpec.get(), other->mScheme) &&
-        SegmentIs(mDirectory, other->mSpec.get(), other->mDirectory) &&
+    // Next check parts of a URI that, if different, automatically make the
+    // URIs different
+    if (!SegmentIs(mScheme, other->mSpec.get(), other->mScheme) ||
+        // Check for host manually, since conversion to file will
+        // ignore the host!
+        !SegmentIs(mHost, other->mSpec.get(), other->mHost) ||
+        !SegmentIs(mQuery, other->mSpec.get(), other->mQuery) ||
+        !SegmentIs(mRef, other->mSpec.get(), other->mRef) ||
+        !SegmentIs(mUsername, other->mSpec.get(), other->mUsername) ||
+        !SegmentIs(mPassword, other->mSpec.get(), other->mPassword) ||
+        Port() != other->Port() ||
+        !SegmentIs(mParam, other->mSpec.get(), other->mParam)) {
+        // No need to compare files or other URI parts -- these are different
+        // beasties
+        *result = PR_FALSE;
+        return NS_OK;
+    }
+    
+    // Then check for exact identity of URIs.  If we have it, they're equal
+    if (SegmentIs(mDirectory, other->mSpec.get(), other->mDirectory) &&
         SegmentIs(mBasename, other->mSpec.get(), other->mBasename) &&
-        SegmentIs(mExtension, other->mSpec.get(), other->mExtension) &&
-        HostsAreEquivalent(other) &&
-        SegmentIs(mQuery, other->mSpec.get(), other->mQuery) &&
-        SegmentIs(mRef, other->mSpec.get(), other->mRef) &&
-        SegmentIs(mUsername, other->mSpec.get(), other->mUsername) &&
-        SegmentIs(mPassword, other->mSpec.get(), other->mPassword) &&
-        SegmentIs(mParam, other->mSpec.get(), other->mParam) &&
-        (Port() == other->Port());
+        SegmentIs(mExtension, other->mSpec.get(), other->mExtension)) {
+        *result = PR_TRUE;
+        return NS_OK;
+    }
+
+    // At this point, the URIs are not identical, but they only differ in the
+    // directory/filename/extension.  If these are file URLs, then get the
+    // corresponding file objects and compare those, since two filenames that
+    // differ, eg, only in case could still be equal.
+    if (mSupportsFileURL) {
+        // Assume not equal for failure cases... but failures in GetFile are
+        // really failures, more or less, so propagate them to caller.
+        *result = PR_FALSE;
+        
+        rv = EnsureFile();
+        if (NS_FAILED(rv)) {
+            LOG(("nsStandardURL::Equals [this=%p spec=%s] failed to ensure file",
+                this, mSpec.get()));
+            return rv;
+        }
+        NS_ASSERTION(mFile, "EnsureFile() lied!");
+        rv = other->EnsureFile();
+        if (NS_FAILED(rv)) {
+            LOG(("nsStandardURL::Equals [other=%p spec=%s] other failed to ensure file",
+                other, other->mSpec.get()));
+            return rv;
+        }
+        NS_ASSERTION(other->mFile, "EnsureFile() lied!");
+        return mFile->Equals(other->mFile, result);
+    }
+
+    // The URLs are not identical, and they do not correspond to the
+    // same file, so they are different.
+    *result = PR_FALSE;
 
-    NS_RELEASE(other);
     return NS_OK;
 }
 
@@ -1479,15 +1619,21 @@ nsStandardURL::SchemeIs(const char *scheme, PRBool *result)
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsStandardURL::Clone(nsIURI **result)
+/* virtual */ nsStandardURL*
+nsStandardURL::StartClone()
 {
     nsStandardURL *clone;
     NS_NEWXPCOM(clone, nsStandardURL);
+    return clone;
+}
+
+NS_IMETHODIMP
+nsStandardURL::Clone(nsIURI **result)
+{
+    nsStandardURL *clone = StartClone();
     if (!clone)
         return NS_ERROR_OUT_OF_MEMORY;
 
-    // XXX a copy-on-write string would be very nice here
     clone->mSpec = mSpec;
     clone->mDefaultPort = mDefaultPort;
     clone->mPort = mPort;
@@ -1702,7 +1848,7 @@ nsStandardURL::GetCommonBaseSpec(nsIURI *uri2, nsACString &aResult)
     nsresult rv = uri2->QueryInterface(kThisImplCID, (void **) &stdurl2);
     isEquals = NS_SUCCEEDED(rv)
             && SegmentIs(mScheme, stdurl2->mSpec.get(), stdurl2->mScheme)    
-            && HostsAreEquivalent(stdurl2)
+            && SegmentIs(mHost, stdurl2->mSpec.get(), stdurl2->mHost)
             && SegmentIs(mUsername, stdurl2->mSpec.get(), stdurl2->mUsername)
             && SegmentIs(mPassword, stdurl2->mSpec.get(), stdurl2->mPassword)
             && (Port() == stdurl2->Port());
@@ -1815,7 +1961,7 @@ nsStandardURL::GetRelativeSpec(nsIURI *uri2, nsACString &aResult)
     while (*thisIndex)
     {
         if (*thisIndex == '/')
-            aResult.Append("../");
+            aResult.AppendLiteral("../");
 
         thisIndex++;
     }
@@ -2244,24 +2390,39 @@ nsStandardURL::SetFileExtension(const nsACString &input)
 // nsStandardURL::nsIFileURL
 //----------------------------------------------------------------------------
 
+nsresult
+nsStandardURL::EnsureFile()
+{
+    NS_PRECONDITION(mSupportsFileURL,
+                    "EnsureFile() called on a URL that doesn't support files!");
+    if (mFile) {
+        // Nothing to do
+        return NS_OK;
+    }
+
+    // Parse the spec if we don't have a cached result
+    if (mSpec.IsEmpty()) {
+        NS_ERROR("url not initialized");
+        return NS_ERROR_NOT_INITIALIZED;
+    }
+
+    if (!SegmentIs(mScheme, "file")) {
+        NS_ERROR("not a file URL");
+        return NS_ERROR_FAILURE;
+    }
+
+    return net_GetFileFromURLSpec(mSpec, getter_AddRefs(mFile));
+}
+
 NS_IMETHODIMP
 nsStandardURL::GetFile(nsIFile **result)
 {
-    // reparse the spec if we don't have a cached result
-    if (!mFile) {
-        if (mSpec.IsEmpty()) {
-            NS_ERROR("url not initialized");
-            return NS_ERROR_NOT_INITIALIZED;
-        }
+    NS_PRECONDITION(mSupportsFileURL,
+                    "GetFile() called on a URL that doesn't support files!");
+    nsresult rv = EnsureFile();
+    if (NS_FAILED(rv))
+        return rv;
 
-        if (!SegmentIs(mScheme, "file")) {
-            NS_ERROR("not a file URL");
-            return NS_ERROR_FAILURE;
-        }
-
-        nsresult rv = net_GetFileFromURLSpec(mSpec, getter_AddRefs(mFile));
-        if (NS_FAILED(rv)) return rv;
-    }
 
 #if defined(PR_LOGGING)
     if (LOG_ENABLED()) {
@@ -2294,7 +2455,14 @@ nsStandardURL::SetFile(nsIFile *file)
     rv = net_GetURLSpecFromFile(file, url);
     if (NS_FAILED(rv)) return rv;
 
-    rv = SetSpec(url);
+    // We should always set the charset until bug 278161 is fixed.
+    nsCOMPtr <nsIPlatformCharset> platformCharset =
+        do_GetService(NS_PLATFORMCHARSET_CONTRACTID);
+    nsCAutoString charset;
+    if (platformCharset)
+        platformCharset->GetCharset(kPlatformCharsetSel_FileName, charset);
+
+    rv = Init(mURLType, mDefaultPort, url, charset.get(), nsnull);
 
     // must clone |file| since its value is not guaranteed to remain constant
     if (NS_SUCCEEDED(rv)) {
@@ -2312,6 +2480,14 @@ nsStandardURL::SetFile(nsIFile *file)
 // nsStandardURL::nsIStandardURL
 //----------------------------------------------------------------------------
 
+inline PRBool
+IsUTFCharset(const char *aCharset)
+{
+    return ((aCharset[0] == 'U' || aCharset[0] == 'u') &&
+            (aCharset[1] == 'T' || aCharset[1] == 't') &&
+            (aCharset[2] == 'F' || aCharset[2] == 'f'));
+}
+
 NS_IMETHODIMP
 nsStandardURL::Init(PRUint32 urlType,
                     PRInt32 defaultPort,
@@ -2340,26 +2516,26 @@ nsStandardURL::Init(PRUint32 urlType,
     mDefaultPort = defaultPort;
     mURLType = urlType;
 
-    if (gAlwaysEncodeInUTF8)
-        mOriginCharset.Truncate();
-    else if (charset == nsnull || *charset == '\0') {
-        mOriginCharset.Truncate();
+    mOriginCharset.Truncate();
+
+    if (charset == nsnull || *charset == '\0') {
         // check if baseURI provides an origin charset and use that.
         if (baseURI)
             baseURI->GetOriginCharset(mOriginCharset);
-    }
-    else
-        mOriginCharset = charset;
 
-    // URI can't be encoded in UTF-16, UTF-16BE, UTF-16LE, UTF-32, UTF-32-LE,
-    // UTF-32LE, UTF-32BE (yet?). Truncate mOriginCharset if it starts with
-    // "utf" (since an empty mOriginCharset implies UTF-8, this is safe even if
-    // mOriginCharset is UTF-8).
-    if (mOriginCharset.Length() >= 3 &&
-        (mOriginCharset[0] == 'U' || mOriginCharset[0] == 'u') &&
-        (mOriginCharset[1] == 'T' || mOriginCharset[1] == 't') &&
-        (mOriginCharset[2] == 'F' || mOriginCharset[2] == 'f'))
-        mOriginCharset.Truncate();
+        // URI can't be encoded in UTF-16, UTF-16BE, UTF-16LE, UTF-32,
+        // UTF-32-LE, UTF-32LE, UTF-32BE (yet?). Truncate mOriginCharset if
+        // it starts with "utf" (since an empty mOriginCharset implies
+        // UTF-8, this is safe even if mOriginCharset is UTF-8).
+
+        if (mOriginCharset.Length() > 3 &&
+            IsUTFCharset(mOriginCharset.get())) {
+            mOriginCharset.Truncate();
+        }
+    }
+    else if (!IsUTFCharset(charset)) {
+        mOriginCharset = charset;
+    }
 
     if (baseURI) {
         PRUint32 start, end;
@@ -2410,8 +2586,10 @@ nsStandardURL::Read(nsIObjectInputStream *stream)
 {
     nsresult rv;
     
-    rv = stream->Read32(&mURLType);
+    PRUint32 urlType;
+    rv = stream->Read32(&urlType);
     if (NS_FAILED(rv)) return rv;
+    mURLType = urlType;
     switch (mURLType) {
       case URLTYPE_STANDARD:
         mParser = net_GetStdURLParser();
